'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mapByKey = exports.groupByDeep = exports.set = exports.get = exports.parsePath = exports.pickBy = exports.toFlatEntries = exports.toFlatObject = exports.toFlatArrayDeep = exports.toFlatArray = exports.getDateRangeArray = exports.isDateRange = exports.createRangeArray = exports.isPromise = exports.isArray = exports.isFunction = exports.isString = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _momentBundle = require('../lib/moment.bundle.js');

var _momentBundle2 = _interopRequireDefault(_momentBundle);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var isString = exports.isString = function isString(arg) {
  return typeof arg === 'string' || arg instanceof String;
};

var isFunction = exports.isFunction = function isFunction(arg) {
  return typeof arg === 'function';
};

var isArray = exports.isArray = function isArray(arg) {
  return Array.isArray(arg);
};

var isPromise = exports.isPromise = function isPromise(arg) {
  return arg != null && isFunction(arg.then);
};

var createRangeArray = exports.createRangeArray = function createRangeArray(start) {
  var end = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : start;
  return Array.from(Array(1 + (end - start)).keys()).map(function (item) {
    return item + start;
  });
};

var isDateRange = exports.isDateRange = function isDateRange(startDate, finalDate) {
  return startDate !== finalDate;
};

var getDateRangeArray = exports.getDateRangeArray = function getDateRangeArray(startDateStr, finalDateStr, dateFormat) {
  var dateRangeDays = [];
  for (var currentDate = (0, _momentBundle2.default)(startDateStr); currentDate <= (0, _momentBundle2.default)(finalDateStr); currentDate.add(1, 'days')) {
    dateRangeDays.push(currentDate.format(dateFormat));
  }

  return dateRangeDays;
};

var toFlatArray = exports.toFlatArray = function toFlatArray(result, arg) {
  return Array.isArray(arg) ? [].concat(_toConsumableArray(result), _toConsumableArray(arg)) : [].concat(_toConsumableArray(result), [arg]);
};

var toFlatArrayDeep = exports.toFlatArrayDeep = function toFlatArrayDeep(result, arg) {
  return Array.isArray(arg) ? [].concat(_toConsumableArray(result), _toConsumableArray(arg.reduce(toFlatArrayDeep, []))) : [].concat(_toConsumableArray(result), [arg]);
};

var toFlatObject = exports.toFlatObject = function toFlatObject(result, obj) {
  return _extends({}, result, obj);
};

var toFlatEntries = exports.toFlatEntries = function toFlatEntries(result, _ref) {
  var _ref2 = _slicedToArray(_ref, 2),
      key = _ref2[0],
      value = _ref2[1];

  return _extends({}, result, _defineProperty({}, key, value));
};

var pickBy = exports.pickBy = function pickBy(obj, fn) {
  return Object.entries(obj).filter(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2),
        key = _ref4[0],
        value = _ref4[1];

    return fn(value, key);
  }).reduce(toFlatEntries, {});
};

var parsePath = exports.parsePath = function parsePath() {
  var path = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  return isString(path) ? path.replace(/['"\]]/g, '').split(/[.[]/g).filter(function (item) {
    return item !== '';
  }) : path;
};

var get = exports.get = function get() {
  var target = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var path = arguments[1];

  var keys = parsePath(path);

  return keys.reduce(function (result, key) {
    return result != null ? result[key] : result;
  }, target);
};

var set = exports.set = function set() {
  var target = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var path = arguments[1];
  var value = arguments[2];

  var keys = parsePath(path);

  var _keys = _toArray(keys),
      key = _keys[0],
      nextKeys = _keys.slice(1);

  var nextValue = nextKeys.length > 0 ? set(target[key], nextKeys, value) : value;

  return _extends({}, target, _defineProperty({}, key, nextValue));
};

var groupByDeep = exports.groupByDeep = function groupByDeep() {
  for (var _len = arguments.length, iteratees = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    iteratees[_key - 1] = arguments[_key];
  }

  var collection = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  var paths = collection.map(function (value) {
    return iteratees.map(function (iteratee) {
      return iteratee(value);
    });
  });

  return paths.reduce(function (result, path, index) {
    var currentValue = get(result, path) || [];
    var newValue = currentValue.concat([collection[index]]);

    return _extends({}, result, set(result, path, newValue));
  }, {});
};

var mapByKey = exports.mapByKey = function mapByKey() {
  var collection = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var key = arguments[1];
  var value = arguments[2];
  return collection.reduce(function (acc, actual) {
    acc[actual[key]] = actual[value];
    return acc;
  }, {});
};