'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.matchesInterfacePartially = exports.matchesInterface = exports.arrayOf = exports.ARRAY_OF = exports.nullable = exports.NULLABLE = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _globalHelper = require('../global/globalHelper.js');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var VALID_PRIMITIVES = [String, Number, Boolean, Array, Object];

var isValidPrimitive = function isValidPrimitive(primitive) {
  return VALID_PRIMITIVES.includes(primitive);
};

var isWrappedAssertion = function isWrappedAssertion(assertion) {
  return !isValidPrimitive(assertion) && assertion !== Function && (0, _globalHelper.isFunction)(assertion);
};

var isValidAssertion = function isValidAssertion(assertion) {
  return isValidPrimitive(assertion) || isWrappedAssertion(assertion);
};

var NULLABLE = exports.NULLABLE = Symbol('NULLABLE');

var nullable = exports.nullable = function nullable(assertion) {
  return function () {
    return {
      type: NULLABLE,
      assertion: assertion
    };
  };
};

var ARRAY_OF = exports.ARRAY_OF = Symbol('ARRAY_OF');

var arrayOf = exports.arrayOf = function arrayOf(primitive) {
  return function () {
    if (!isValidPrimitive(primitive)) {
      throw new Error('Invalid parameter: "' + primitive + '". It should be one of String, ' + 'Number, Boolean, Array or Object primitives.');
    }

    return {
      type: ARRAY_OF,
      primitive: primitive
    };
  };
};

var isNullable = function isNullable(arg) {
  return isWrappedAssertion(arg) && arg().type === NULLABLE;
};

var checkMatch = function checkMatch(value, assertion) {
  var type = isWrappedAssertion(assertion) ? assertion().type : undefined;

  var primitive = isWrappedAssertion(assertion) ? assertion().primitive : assertion;

  switch (type) {
    case ARRAY_OF:
      return Array.isArray(value) && value.every(function (item) {
        return checkMatch(item, primitive);
      });

    default:
      return value.constructor === primitive;
  }
};

var checkMatchWith = function checkMatchWith(obj) {
  return function (result, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        assertion = _ref2[1];

    if (!isValidAssertion(assertion)) {
      throw new Error('Invalid assertion: "' + assertion + '".');
    }

    var isNullableAssertion = isNullable(assertion);
    var parsedAssertion = assertion;

    if (isNullable(assertion)) {
      parsedAssertion = assertion().assertion;
    }

    var partial = isNullableAssertion ? obj[key] == null || checkMatch(obj[key], parsedAssertion) : obj[key] != null && checkMatch(obj[key], parsedAssertion);

    return [].concat(_toConsumableArray(result), [partial]);
  };
};

var makeEntryNullable = function makeEntryNullable(_ref3) {
  var _ref4 = _slicedToArray(_ref3, 2),
      key = _ref4[0],
      value = _ref4[1];

  var nullableValue = isNullable(value) ? value : nullable(value);
  return [key, nullableValue];
};

var matchesInterfaceWithMap = function matchesInterfaceWithMap(mapFunc) {
  return function (interfc) {
    return function (obj) {
      var entriesMatch = Object.entries(interfc).map(mapFunc).reduce(checkMatchWith(obj), []).every(function (item) {
        return item === true;
      });

      var hasSameKeys = Object.keys(obj).every(function (objKey) {
        return interfc[objKey] != null;
      });

      return entriesMatch && hasSameKeys;
    };
  };
};

var matchesInterface = exports.matchesInterface = matchesInterfaceWithMap(function (entry) {
  return entry;
});

var matchesInterfacePartially = exports.matchesInterfacePartially = matchesInterfaceWithMap(makeEntryNullable);